--- old/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas.c	2014-12-22 09:51:44.315386971 -0500
+++ new/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas.c	2014-12-22 09:51:44.326369496 -0500
@@ -9854,6 +9854,26 @@
 				    "after fault was detected");
 			}
 		}
+		
+		/*
+		 * If we set the "please reset me" flag, then reset.
+		 */
+		if (mpt->m_softstate & MPTSAS_SS_RESET_INWATCH) {
+			doorbell = ddi_get32(mpt->m_datap,
+			    &mpt->m_reg->Doorbell);
+			mptsas_log(mpt, CE_WARN, "MPT Forced Reset, "
+			    "doorbell: %04x", doorbell);
+			mpt->m_softstate &= ~MPTSAS_SS_MSG_UNIT_RESET;
+			if (mpt->m_softstate & MPTSAS_SS_MUR_INWATCH)
+				mpt->m_softstate |= MPTSAS_SS_MSG_UNIT_RESET;
+			mpt->m_softstate &= ~(MPTSAS_SS_RESET_INWATCH|
+			    MPTSAS_SS_MUR_INWATCH);
+			if ((mptsas_restart_ioc(mpt)) == DDI_FAILURE) {
+				mptsas_log(mpt, CE_WARN, "Reset failed"
+				    "after fault was detected");
+			}
+		}
+		
 
 		/*
 		 * For now, always call mptsas_watchsubr.
--- old/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas_impl.c	2014-12-22 09:51:45.254304366 -0500
+++ new/usr/src/uts/common/io/scsi/adapters/mpt_sas/mptsas_impl.c	2014-12-22 09:51:45.282654757 -0500
@@ -1196,19 +1196,17 @@
 	slots->m_slot[MPTSAS_TM_SLOT(mpt)] = NULL;
 
 	/*
-	 * If we lost our task management command
-	 * we need to reset the ioc
+	 * If we lost our task management command we need to reset the ioc
+	 * but we can't do it here as it's most likely we were called from
+	 * the taskq that mptsas_restart_ioc()->mptsas_flush_hba() needs to
+	 * wait for. So set a flag for mptsas_watch().
 	 */
 	if (rval == FALSE) {
 		mptsas_log(mpt, CE_WARN, "mptsas_ioc_task_management failed "
-		    "try to reset ioc to recovery!");
-		mpt->m_softstate &= ~MPTSAS_SS_MSG_UNIT_RESET;
-		if (mptsas_restart_ioc(mpt)) {
-			mptsas_log(mpt, CE_WARN, "mptsas_restart_ioc failed");
-			rval = FAILED;
-		}
+		    "schedule reset in watch!");
+		mpt->m_softstate |= MPTSAS_SS_RESET_INWATCH;
 	}
-
+	
 	return (rval);
 }
 
--- old/usr/src/uts/common/sys/scsi/adapters/mpt_sas/mptsas_var.h	2014-12-22 09:51:45.596473082 -0500
+++ new/usr/src/uts/common/sys/scsi/adapters/mpt_sas/mptsas_var.h	2014-12-22 09:51:45.624552957 -0500
@@ -1016,6 +1016,8 @@
 #define	MPTSAS_SS_QUIESCED		0x04
 #define	MPTSAS_SS_MSG_UNIT_RESET	0x08
 #define	MPTSAS_DID_MSG_UNIT_RESET	0x10
+#define MPTSAS_SS_RESET_INWATCH         0x20
+#define MPTSAS_SS_MUR_INWATCH           0x40
 
 /*
  * regspec defines.
